@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2008-2020 The Octave Project Developers
+## Copyright (C) 2008-2023 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -24,8 +24,12 @@
 ########################################################################
 
 ## -*- texinfo -*-
-## @deftypefn  {} {} fminunc (@var{fcn}, @var{x0})
-## @deftypefnx {} {} fminunc (@var{fcn}, @var{x0}, @var{options})
+## @deftypefn  {} {@var{x} =} fminunc (@var{fcn}, @var{x0})
+## @deftypefnx {} {@var{x} =} fminunc (@var{fcn}, @var{x0}, @var{options})
+## @deftypefnx {} {[@var{x}, @var{fval}] =} fminunc (@var{fcn}, @dots{})
+## @deftypefnx {} {[@var{x}, @var{fval}, @var{info}] =} fminunc (@var{fcn}, @dots{})
+## @deftypefnx {} {[@var{x}, @var{fval}, @var{info}, @var{output}] =} fminunc (@var{fcn}, @dots{})
+## @deftypefnx {} {[@var{x}, @var{fval}, @var{info}, @var{output}, @var{grad}] =} fminunc (@var{fcn}, @dots{})
 ## @deftypefnx {} {[@var{x}, @var{fval}, @var{info}, @var{output}, @var{grad}, @var{hess}] =} fminunc (@var{fcn}, @dots{})
 ## Solve an unconstrained optimization problem defined by the function
 ## @var{fcn}.
@@ -33,7 +37,7 @@
 ## @code{fminunc} attempts to determine a vector @var{x} such that
 ## @code{@var{fcn} (@var{x})} is a local minimum.
 ##
-## @var{fun} is a function handle, inline function, or string containing the
+## @var{fcn} is a function handle, inline function, or string containing the
 ## name of the function to evaluate.  @var{fcn} should accept a vector (array)
 ## defining the unknown variables, and return the objective function value,
 ## optionally with gradient.
@@ -70,7 +74,8 @@
 ## @var{x}, while @qcode{"TolFun"} is a tolerance for the objective function
 ## value @var{fval}.  The default is @code{1e-6} for both options.
 ##
-## For a description of the other options, see @code{optimset}.
+## For a description of the other options,
+## @pxref{XREFoptimset,,@code{optimset}}.
 ##
 ## On return, @var{x} is the location of the minimum and @var{fval} contains
 ## the value of the objective function at @var{x}.
@@ -128,7 +133,7 @@ function [x, fval, info, output, grad, hess] = fminunc (fcn, x0, options = struc
     return;
   endif
 
-  if (nargin < 2 || nargin > 3 || ! isnumeric (x0))
+  if (nargin < 2 || ! isnumeric (x0))
     print_usage ();
   endif
 
@@ -408,12 +413,12 @@ function [x, fval, info, output, grad, hess] = fminunc (fcn, x0, options = struc
 endfunction
 
 ## A helper function that evaluates a function and checks for bad results.
-function [fx, gx] = guarded_eval (fun, x)
+function [fx, gx] = guarded_eval (fcn, x)
 
   if (nargout > 1)
-    [fx, gx] = fun (x);
+    [fx, gx] = fcn (x);
   else
-    fx = fun (x);
+    fx = fcn (x);
     gx = [];
   endif
 
