@@ -1,20 +1,27 @@
-## Copyright (C) 1994-2017 John W. Eaton
+########################################################################
+##
+## Copyright (C) 1994-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {} axis ()
@@ -32,9 +39,9 @@
 ## first and second elements specify the lower and upper limits for the
 ## x-axis.  The third and fourth specify the limits for the y-axis, the fifth
 ## and sixth specify the limits for the z-axis, and the seventh and eighth
-## specify the limits for the color axis.  The special values -Inf and Inf may
-## be used to indicate that the limit should be automatically computed based
-## on the data in the axis.
+## specify the limits for the color axis.  The special values @code{-Inf} and
+## @code{Inf} may be used to indicate that the limit should be automatically
+## computed based on the data in the axes.
 ##
 ## Without any arguments, @code{axis} turns autoscaling on.
 ##
@@ -47,14 +54,20 @@
 ## The following options control the aspect ratio of the axes.
 ##
 ## @table @asis
-## @item @qcode{"square"}
-## Force a square axis aspect ratio.
-##
 ## @item @qcode{"equal"}
 ## Force x-axis unit distance to equal y-axis (and z-axis) unit distance.
 ##
-## @item @qcode{"normal"}
-## Restore default aspect ratio.
+## @item @qcode{"square"}
+## Force a square axis aspect ratio.
+##
+## @item @nospell{@qcode{"vis3d"}}
+## Set aspect ratio modes (@qcode{"DataAspectRatio"},
+## @qcode{"PlotBoxAspectRatio"}) to @qcode{"manual"} for rotation without
+## stretching.
+##
+## @item  @qcode{"normal"}
+## @itemx @qcode{"fill"}
+## Restore default automatically computed aspect ratios.
 ## @end table
 ##
 ## @noindent
@@ -63,28 +76,41 @@
 ## @table @asis
 ## @item  @qcode{"auto"}
 ## @itemx @qcode{"auto[xyz]"}
-## Set the specified axes to have nice limits around the data or all if no
-## axes are specified.
+## @itemx @qcode{"auto [xyz]"}
+## Set nice auto-computed limits around the data for all axes, or only
+## the specified axes.
 ##
 ## @item @qcode{"manual"}
 ## Fix the current axes limits.
 ##
+## @item @qcode{"tickaligned"}
+## Fix axes to the limits of the closest ticks.
+##
 ## @item @qcode{"tight"}
 ## Fix axes to the limits of the data.
 ##
+## @item @qcode{"padded"}
+## Fix axes to the limits of the data plus margins of about 7% of the
+## data extent.
+##
 ## @item @qcode{"image"}
 ## Equivalent to @qcode{"tight"} and @qcode{"equal"}.
+##
 ## @end table
 ##
 ## @noindent
 ## The following options affect the appearance of tick marks.
 ##
 ## @table @asis
-## @item @qcode{"tic[xyz]"}
+## @item  @qcode{"tic"}
+## @itemx @qcode{"tic[xyz]"}
+## @itemx @qcode{"tic [xyz]"}
 ## Turn tick marks on for all axes, or turn them on for the specified axes and
 ## off for the remainder.
 ##
-## @item @qcode{"label[xyz]"}
+## @item  @qcode{"label"}
+## @itemx @qcode{"label[xyz]"}
+## @itemx @qcode{"label [xyz]"}
 ## Turn tick labels on for all axes, or turn them on for the specified axes
 ## and off for the remainder.
 ##
@@ -92,7 +118,7 @@
 ## Turn tick labels off for all axes.
 ## @end table
 ##
-## Note: If there are no tick marks for an axis then there can be no labels.
+## Note: If there are no tick marks for an axes then there can be no labels.
 ##
 ## @noindent
 ## The following options affect the direction of increasing values on the axes.
@@ -135,8 +161,6 @@
 ## @seealso{xlim, ylim, zlim, caxis, daspect, pbaspect, box, grid}
 ## @end deftypefn
 
-## Author: jwe
-
 function limits = axis (varargin)
 
   [hax, varargin, nargin] = __plt_get_axis_arg__ ("axis", varargin{:});
@@ -162,7 +186,7 @@ function limits = axis (varargin)
 
 endfunction
 
-function limits = __axis__ (ca, ax, varargin)
+function limits = __axis__ (ca, varargin)
 
   if (nargin == 1)
     if (nargout == 0)
@@ -178,245 +202,202 @@ function limits = __axis__ (ca, ax, varargin)
         limits = [xlim, ylim, zlim];
       endif
     endif
+    return;
+  endif
 
-  elseif (ischar (ax))
-    len = length (ax);
+  for arg = varargin
+    opt = arg{1};
 
-    ## 'matrix mode' to reverse the y-axis
-    if (strcmpi (ax, "ij"))
-      set (ca, "ydir", "reverse");
-    elseif (strcmpi (ax, "xy"))
-      set (ca, "ydir", "normal");
+    if (ischar (opt))
+      len = length (opt);
+
+      ## 'matrix mode' to reverse the y-axis
+      if (strcmpi (opt, "ij"))
+        set (ca, "ydir", "reverse");
+      elseif (strcmpi (opt, "xy"))
+        set (ca, "ydir", "normal");
 
       ## aspect ratio
-    elseif (strcmpi (ax, "image"))
-      __axis__ (ca, "equal");
-      set (ca, "plotboxaspectratiomode", "auto");
-      __do_tight_option__ (ca);
-    elseif (strcmpi (ax, "square"))
-      set (ca, "dataaspectratiomode", "auto",
-               "plotboxaspectratio", [1, 1, 1]);
-    elseif (strcmp (ax, "equal"))
-      ## Get position of axis in pixels
-      ca_units = get (ca, "units");
-      set (ca, "units", "pixels");
-      axis_pos = get (ca, "position");
-      set (ca, "units", ca_units);
+      elseif (strcmpi (opt, "image"))
+        __axis__ (ca, "equal");
+        set (ca, "plotboxaspectratiomode", "auto", ...
+                 "xlimmode", "auto", "ylimmode", "auto", ...
+                 "zlimmode", "auto", ...
+                 "xlimitmethod", "tight", "ylimitmethod", "tight");
+      elseif (strcmpi (opt, "square"))
+        set (ca, "dataaspectratiomode", "auto",
+                 "plotboxaspectratio", [1, 1, 1]);
+      elseif (strcmp (opt, "equal"))
+        is2dview = (get (ca, "view")(2) == 90);
+        if (is2dview)
+          ## Save & later restore axes aspect ratio
+          ca_units = get (ca, "units");
+          set (ca, "units", "pixels");
+          axis_pos = get (ca, "position");
+          set (ca, "units", ca_units);
+          pbratio = [axis_pos(3), axis_pos(4), axis_pos(4)];
+        endif
 
-      pbar = get (ca, "PlotBoxAspectRatio");
-      dx = diff (__get_tight_lims__ (ca, "x"));
-      dy = diff (__get_tight_lims__ (ca, "y"));
-      dz = diff (__get_tight_lims__ (ca, "z"));
-      new_pbar = [dx dy dz];
-      new_pbar(new_pbar == 0) = 1;
-      if (dx/pbar(1) < dy/pbar(2))
-        set (ca, "xlimmode", "auto");
-        new_pbar(1) = dy / axis_pos(4)*axis_pos(3);
-      else
-        set (ca, "ylimmode", "auto");
-        new_pbar(2) = dx / axis_pos(3)*axis_pos(4);
-      endif
-      set (ca, "dataaspectratio", [1, 1, 1],
-               "plotboxaspectratio", new_pbar);
+        daratio = get (ca, "dataaspectratio");
+        if (all (daratio != [1, 1, 1]))
+          set (ca, "dataaspectratio", [1, 1, 1]);
+        else
+          set (ca, "dataaspectratio", [1+eps, 1, 1],
+                   "dataaspectratio", [1, 1, 1]);
+        endif
 
-    elseif (strcmpi (ax, "normal"))
-      ## Set plotboxaspectratio to something obtuse so that switching
-      ## back to "auto" will force a re-calculation.
-      set (ca, "plotboxaspectratio", [3 2 1]);
-      set (ca, "plotboxaspectratiomode", "auto",
-               "dataaspectratiomode", "auto");
+        ## Matlab only adjusts plotboxaspectratio for 2-D figures, but
+        ## Octave needs to do it for 3-D as well.
+        if (is2dview)
+          set (ca, "plotboxaspectratio", pbratio);
+        else
+          pbratio = get (ca, "plotboxaspectratio");
+          if (all (pbratio != [1, 1, 1]))
+            set (ca, "plotboxaspectratio", [1, 1, 1]);
+          else
+            set (ca, "plotboxaspectratio", [1+eps, 1, 1],
+                     "plotboxaspectratio", [1, 1, 1]);
+          endif
+        endif
+      elseif (strcmpi (opt, "vis3d"))
+        ## Fix aspect ratio modes for rotation without stretching.
+        set (ca, "dataaspectratiomode", "manual",
+                 "plotboxaspectratiomode", "manual");
+      elseif (strcmpi (opt, "normal") || strcmpi (opt, "fill"))
+        ## Set plotboxaspectratio to something obtuse so that switching
+        ## back to "auto" will force a re-calculation.
+        set (ca, "plotboxaspectratio", [3 2 1]);
+        set (ca, "plotboxaspectratiomode", "auto",
+                 "dataaspectratiomode", "auto");
 
       ## axis limits
-    elseif (len >= 4 && strcmpi (ax(1:4), "auto"))
-      if (len > 4)
-        if (any (ax == "x"))
-          set (ca, "xlimmode", "auto");
+      elseif (strncmpi (opt, "auto", 4))
+        if (len == 4)
+          set (ca, "xlimmode", "auto", "ylimmode", "auto", "zlimmode", "auto");
+        else
+          if (any (opt == "x"))
+            set (ca, "xlimmode", "auto");
+          endif
+          if (any (opt == "y"))
+            set (ca, "ylimmode", "auto");
+          endif
+          if (any (opt == "z"))
+            set (ca, "zlimmode", "auto");
+          endif
         endif
-        if (any (ax == "y"))
-          set (ca, "ylimmode", "auto");
-        endif
-        if (any (ax == "z"))
-          set (ca, "zlimmode", "auto");
-        endif
-      else
-        set (ca, "xlimmode", "auto", "ylimmode", "auto", "zlimmode", "auto");
-      endif
-    elseif (strcmpi (ax, "manual"))
-      ## fixes the axis limits, like axis(axis) should;
-      set (ca, "xlimmode", "manual", "ylimmode", "manual", "zlimmode", "manual");
-    elseif (strcmpi (ax, "tight"))
-      ## sets the axis limits to the min and max of all data.
-      __do_tight_option__ (ca);
+      elseif (strcmpi (opt, "manual"))
+        ## fixes the axis limits
+        set (ca, "xlimmode", "manual", "ylimmode", "manual",
+                 "zlimmode", "manual");
+      elseif (strcmpi (opt, "tickaligned"))
+        ## sets the axis limits to closest ticks.
+        set (ca, "xlimmode", "auto", "ylimmode", "auto", ...
+                 "zlimmode", "auto", ...
+                 "xlimitmethod", "tickaligned", ...
+                 "ylimitmethod", "tickaligned", ...
+                 "zlimitmethod", "tickaligned");
+      elseif (strcmpi (opt, "tight"))
+        ## sets the axis limits to the min and max of all data.
+        set (ca, "xlimmode", "auto", "ylimmode", "auto", ...
+                 "zlimmode", "auto", ...
+                 "xlimitmethod", "tight", "ylimitmethod", "tight",
+                 "zlimitmethod", "tight");
+      elseif (strcmpi (opt, "padded"))
+        ## sets the axis limits to the min and max of all data plus padding.
+        set (ca, "xlimmode", "auto", "ylimmode", "auto", ...
+                 "zlimmode", "auto", ...
+                 "xlimitmethod", "padded", ...
+                 "ylimitmethod", "padded", ...
+                 "zlimitmethod", "padded");
+      ## visibility
+      elseif (strcmpi (opt, "on"))
+        set (ca, "visible", "on");
+      elseif (strcmpi (opt, "off"))
+        set (ca, "visible", "off");
+
       ## tick marks
-    elseif (strcmpi (ax, "on") || strcmpi (ax, "tic"))
-      set (ca, "xtickmode", "auto", "ytickmode", "auto", "ztickmode", "auto");
-      if (strcmpi (ax, "on"))
+      elseif (strcmpi (opt, "tic"))
+        set (ca, "xtickmode", "auto", "ytickmode", "auto", "ztickmode", "auto",
+                 "visible", "on");
+      elseif (strncmpi (opt, "tic", 3))
+        if (any (opt == "x"))
+          set (ca, "xtickmode", "auto");
+        else
+          set (ca, "xtick", []);
+        endif
+        if (any (opt == "y"))
+          set (ca, "ytickmode", "auto");
+        else
+          set (ca, "ytick", []);
+        endif
+        if (any (opt == "z"))
+          set (ca, "ztickmode", "auto");
+        else
+          set (ca, "ztick", []);
+        endif
+
+      ## labels
+      elseif (strcmpi (opt, "label"))
         set (ca, "xticklabelmode", "auto", "yticklabelmode", "auto",
-           "zticklabelmode", "auto");
-      endif
-      set (ca, "visible", "on");
-    elseif (strcmpi (ax, "off"))
-      set (ca, "xtick", [], "ytick", [], "ztick", []);
-      set (ca, "visible", "off");
-    elseif (len > 3 && strcmpi (ax(1:3), "tic"))
-      if (any (ax == "x"))
-        set (ca, "xtickmode", "auto");
+                 "zticklabelmode", "auto");
+      elseif (strcmpi (opt, "nolabel"))
+        set (ca, "xticklabel", {}, "yticklabel", {}, "zticklabel", {})
+      elseif (strncmpi (opt, "label", 5))
+        if (any (opt == "x"))
+          set (ca, "xticklabelmode", "auto");
+        else
+          set (ca, "xticklabel", "");
+        endif
+        if (any (opt == "y"))
+          set (ca, "yticklabelmode", "auto");
+        else
+          set (ca, "yticklabel", "");
+        endif
+        if (any (opt == "z"))
+          set (ca, "zticklabelmode", "auto");
+        else
+          set (ca, "zticklabel", "");
+        endif
+
       else
-        set (ca, "xtick", []);
+        warning ("axis: unknown option '%s'", opt);
       endif
-      if (any (ax == "y"))
-        set (ca, "ytickmode", "auto");
-      else
-        set (ca, "ytick", []);
+
+    elseif (isnumeric (opt) && isvector (opt))
+
+      len = length (opt);
+
+      if (len != 2 && len != 4 && len != 6 && len != 8)
+        error ("axis: LIMITS vector must have 2, 4, 6, or 8 elements");
       endif
-      if (any (ax == "z"))
-        set (ca, "ztickmode", "auto");
-      else
-        set (ca, "ztick", []);
+
+      for i = 1:2:len
+        if (opt(i) >= opt(i+1))
+          error ("axis: LIMITS(%d) must be less than LIMITS(%d)", i, i+1);
+        endif
+      endfor
+
+      if (len > 1)
+        xlim (ca, opt(1:2));
       endif
-    elseif (strcmpi (ax, "label"))
-      set (ca, "xticklabelmode", "auto", "yticklabelmode", "auto",
-           "zticklabelmode", "auto");
-    elseif (strcmpi (ax, "nolabel"))
-      set (ca, "xticklabel", "", "yticklabel", "", "zticklabel", "");
-    elseif (len > 5 && strcmpi (ax(1:5), "label"))
-      if (any (ax == "x"))
-        set (ca, "xticklabelmode", "auto");
-      else
-        set (ca, "xticklabel", "");
+
+      if (len > 3)
+        ylim (ca, opt(3:4));
       endif
-      if (any (ax == "y"))
-        set (ca, "yticklabelmode", "auto");
-      else
-        set (ca, "yticklabel", "");
+
+      if (len > 5)
+        zlim (ca, opt(5:6));
       endif
-      if (any (ax == "z"))
-        set (ca, "zticklabelmode", "auto");
-      else
-        set (ca, "zticklabel", "");
+
+      if (len > 7)
+        caxis (ca, opt(7:8));
       endif
 
     else
-      warning ("axis: unknown option '%s'", ax);
+      error ("axis: expecting no args, or a numeric vector with 2, 4, 6, or 8 elements");
     endif
 
-  elseif (isnumeric (ax) && isvector (ax))
-
-    len = length (ax);
-
-    if (len != 2 && len != 4 && len != 6 && len != 8)
-      error ("axis: LIMITS vector must have 2, 4, 6, or 8 elements");
-    endif
-
-    for i = 1:2:len
-      if (ax(i) >= ax(i+1))
-        error ("axis: LIMITS(%d) must be less than LIMITS(%d)", i, i+1);
-      endif
-    endfor
-
-    if (len > 1)
-      xlim (ca, ax(1:2));
-    endif
-
-    if (len > 3)
-      ylim (ca, ax(3:4));
-    endif
-
-    if (len > 5)
-      zlim (ca, ax(5:6));
-    endif
-
-    if (len > 7)
-      caxis (ca, ax(7:8));
-    endif
-
-  else
-    error ("axis: expecting no args, or a numeric vector with 2, 4, 6, or 8 elements");
-  endif
-
-  if (! isempty (varargin))
-    __axis__ (ca, varargin{:});
-  endif
-
-endfunction
-
-function lims = __get_tight_lims__ (ca, ax)
-
-  ## Get the limits for axis ("tight").
-  ## AX should be one of "x", "y", or "z".
-  kids = findobj (ca, "-property", [ax "data"]);
-  ## The data properties for hggroups mirror their children.
-  ## Exclude the redundant hgroup values.
-  hg_kids = findobj (kids, "type", "hggroup");
-  kids = setdiff (kids, hg_kids);
-  if (isempty (kids))
-    ## Return the current limits.
-    lims = get (ca, [ax "lim"]);
-  else
-    data = get (kids, [ax "data"]);
-    types = get (kids, "type");
-
-    scale = get (ca, [ax "scale"]);
-    if (! iscell (data))
-      data = {data};
-    endif
-
-    ## Extend image data one pixel
-    idx = strcmp (types, "image");
-    if (any (idx) && (ax == "x" || ax == "y"))
-      imdata = data(idx);
-      px = arrayfun (@__image_pixel_size__, kids(idx), "uniformoutput", false);
-      ipx = ifelse (ax == "x", 1, 2);
-      imdata = cellfun (@(x,dx) [(min (x) - dx(ipx)), (max (x) + dx(ipx))],
-                        imdata, px, "uniformoutput", false);
-      data(idx) = imdata;
-    endif
-
-    if (strcmp (scale, "log"))
-      tmp = data;
-      data = cellfun (@(x) x(x>0), tmp, "uniformoutput", false);
-      n = cellfun ("isempty", data);
-      data(n) = cellfun (@(x) x(x<0), tmp(n), "uniformoutput", false);
-    endif
-    data = cellfun (@(x) x(isfinite (x)), data, "uniformoutput", false);
-    data = data(! cellfun ("isempty", data));
-    if (! isempty (data))
-      ## Change data from cell array of various sizes to a single column vector
-      data = cat (1, cellindexmat (data, ":"){:});
-      lims = [min(data), max(data)];
-    else
-      lims = [0, 1];
-    endif
-  endif
-
-endfunction
-
-function __do_tight_option__ (ca)
-
-  xlim = __get_tight_lims__ (ca, "x");
-  if (all (xlim == 0))
-    xlim = eps () * [-1 1];
-  elseif (diff (xlim == 0))
-    xlim .*= (1 + eps () * [-1, 1]);
-  endif
-  ylim = __get_tight_lims__ (ca, "y");
-  if (all (ylim == 0))
-    ylim = eps () * [-1 1];
-  elseif (diff (ylim == 0))
-    ylim .*= (1 + eps () * [-1, 1]);
-  endif
-  set (ca, "xlim", xlim, "ylim", ylim);
-  nd = __calc_dimensions__ (ca);
-  is3dview = (get (ca, "view")(2) != 90);
-  if (nd > 2 && is3dview)
-    zlim = __get_tight_lims__ (ca, "z");
-    if (all (zlim == 0))
-      zlim = eps () * [-1 1];
-    elseif (diff (zlim == 0))
-      zlim .*= (1 + eps () * [-1, 1]);
-    endif
-    set (ca, "zlim", zlim);
-  endif
+  endfor
 
 endfunction
 
@@ -465,6 +446,8 @@ endfunction
 
 %!demo
 %! clf;
+%! def_ticklen = get (0, "DefaultAxesTickLength");
+%! set (0, "DefaultAxesTickLength", [.028, .025]);
 %! t = 0:0.01:2*pi;
 %! x = sin (t);
 %!
@@ -512,6 +495,8 @@ endfunction
 %!  plot (t, x);
 %!  title ("all ticks and labels");
 %!  axis ("on");
+%!
+%! set (0, "DefaultAxesTickLength", def_ticklen);
 
 %!demo
 %! clf;
@@ -547,14 +532,14 @@ endfunction
 %!  axis ("autoy");
 %!
 %! subplot (326);
-%!  plot (t, sin(t), t, -2*sin(t/2));
+%!  plot (t, sin (t), t, -2*sin (t/2));
 %!  axis ("tight");
 %!  title ("tight");
 
 %!demo
 %! clf;
 %! x = 0:0.1:10;
-%! plot (x, sin(x));
+%! plot (x, sin (x));
 %! axis image;
 %! title ({"image", 'equivalent to "tight" & "equal"'});
 
@@ -569,70 +554,45 @@ endfunction
 %! s = x1 / max (x(:));
 %! pcolor (s*x+x1, s*y+x1/2, 5*z);
 %! axis tight;
-
-%!demo
-%! clf;
-%! x = -10:10;
-%! plot (x,x, x,-x);
-%! set (gca, "yscale", "log");
-%! legend ({"x >= 1", "x <= 1"}, "location", "north");
-%! title ("ylim = [1, 10]");
+%! title ('"tight" when two axes objects exist');
 
 %!demo
 %! clf;
 %! loglog (1:20, "-s");
 %! axis tight;
+%! title ('"tight" on loglog plot');
 
 %!demo
 %! clf;
-%! x = -10:0.1:10;
-%! y = sin (x)./(1 + abs (x)) + 0.1*x - 0.4;
-%! plot (x, y);
-%! set (gca, "xaxislocation", "origin");
-%! set (gca, "yaxislocation", "origin");
-%! box off;
-%! title ({"no plot box", "xaxislocation = origin, yaxislocation = origin"});
+%! x = y = 0.5:0.5:12;
+%! subplot (3,1,1);
+%! plot (x, y, "-s");
+%! axis tickaligned
+%! title ("tickaligned");
+%! subplot (3,1,2);
+%! plot (x, y, "-s");
+%! axis padded
+%! title ("padded");
+%! subplot (3,1,3);
+%! plot (x, y, "-s");
+%! axis tight
+%! title ("tight");
 
 %!demo
 %! clf;
-%! x = -10:0.1:10;
-%! y = sin (x)./(1+abs (x)) + 0.1*x - 0.4;
-%! plot (x, y);
-%! set (gca, "xaxislocation", "origin");
-%! set (gca, "yaxislocation", "left");
-%! box off;
-%! title ({"no plot box", "xaxislocation = origin, yaxislocation = left"});
-
-%!demo
-%! clf;
-%! x = -10:0.1:10;
-%! y = sin (x)./(1+abs (x)) + 0.1*x - 0.4;
-%! plot (x, y);
-%! title ("no plot box");
-%! set (gca, "xaxislocation", "origin");
-%! set (gca, "yaxislocation", "right");
-%! box off;
-%! title ({"no plot box", "xaxislocation = origin, yaxislocation = right"});
-
-%!demo
-%! clf;
-%! x = -10:0.1:10;
-%! y = sin (x)./(1+abs (x)) + 0.1*x - 0.4;
-%! plot (x, y);
-%! set (gca, "xaxislocation", "bottom");
-%! set (gca, "yaxislocation", "origin");
-%! box off;
-%! title ({"no plot box", "xaxislocation = bottom, yaxislocation = origin"});
-
-%!demo
-%! clf;
-%! x = -10:0.1:10;
-%! y = sin (x)./(1+abs (x)) + 0.1*x - 0.4;
-%! plot (x, y);
-%! set (gca, "xaxislocation", "top");
-%! set (gca, "yaxislocation", "origin");
-%! box off;
-%! title ({"no plot box", "xaxislocation = top, yaxislocation = origin"});
+%! x = y = 0.5:0.5:12;
+%! subplot (3,1,1);
+%! loglog (x, y, "-s");
+%! axis tickaligned
+%! title ("tickaligned");
+%! subplot (3,1,2);
+%! loglog (x, y, "-s");
+%! axis padded
+%! title ("padded");
+%! subplot (3,1,3);
+%! loglog (x, y, "-s");
+%! axis tight
+%! title ("tight");
 
 %!test
 %! hf = figure ("visible", "off");
@@ -652,13 +612,15 @@ endfunction
 %!   a = logspace (-5, 1, 10);
 %!   loglog (a, -a);
 %!   axis tight;
-%!   assert (axis (), [1e-5, 10, -10, -1e-5]);
+%!   ## Compare to powers of 10 rather than exponential literals because of an
+%!   ## upstream bug in mingw-w64: https://sourceforge.net/p/mingw-w64/bugs/466/
+%!   assert (axis (), [10^-5, 10, -10, -10^-5]);
 %! unwind_protect_cleanup
 %!   close (hf);
 %! end_unwind_protect
 
 ## Test 'axis tight' with differently oriented, differently numbered data vecs
-%!test <40036>
+%!test <*40036>
 %! hf = figure ("visible", "off");
 %! unwind_protect
 %!   Z = peaks (linspace (-3, 3, 49), linspace (-2, 2, 29));
@@ -669,8 +631,21 @@ endfunction
 %!   close (hf);
 %! end_unwind_protect
 
-## Even on errors, axis can display a figure.
+## Test 'axis tight' remains after addition of new data
+%!test
+%! hf = figure ("visible", "off");
+%! unwind_protect
+%!   plot (1:10)
+%!   axis tight;
+%!   assert (axis (), [1 10 1 10]);
+%!   hold on;
+%!   plot (1:11)
+%!   assert (axis (), [1 11 1 11]);
+%! unwind_protect_cleanup
+%!   close (hf);
+%! end_unwind_protect
 
+## Even on errors, axis can display a figure.
 %!error <LIMITS vector must have .* elements>
 %! hf = figure ("visible", "off");
 %! unwind_protect
@@ -679,6 +654,14 @@ endfunction
 %!   close (hf);
 %! end_unwind_protect
 
+%!error <LIMITS\(3\) must be less than LIMITS\(4\)>
+%! hf = figure ("visible", "off");
+%! unwind_protect
+%!   axis ([1 2 4 3])
+%! unwind_protect_cleanup
+%!   close (hf);
+%! end_unwind_protect
+
 %!error <expecting no args, or a numeric vector with .* elements>
 %! hf = figure ("visible", "off");
 %! unwind_protect
